package com.bolour.sample.eclipse.listener.subject;

import java.util.Enumeration;
import java.util.Vector;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;

/**
 * Sample <i>subject</i> plug-in for the <i>listener</i> extension pattern.
 * The subject plug-in provides a <code>listeners</code> extension-point, and 
 * notifes the extension members of this extension-point when its state is updated
 * via the <code>update</code> method. 
 * <p>
 * Extension processing by this plug-in obtains references to its listeners
 * from all of its extensions, and saves these references for future notification.
 * When the state of the plugin is changed through the <code>update</code> method, 
 * these listeners are notified.
 * <p>
 * The plug-in class is used as a natural place to house the 
 * extension and notification processing. No specific activation/deactivation 
 * processing is required by this plug-in. So strictly speaking, a plug-in
 * class is not necessary for this plug-in. Nevertheless, the plug-in instance
 * provides a convenient place to do the basic work required of this plug-in.
 * 
 * @author Azad
 */
public class Subject extends Plugin {
	
	private static final String SUBJECT_PLUGIN = 
	  "com.bolour.sample.eclipse.listener.subject";
	
	private static final String LISTENERS_EXTENSION_POINT = 
	  "com.bolour.sample.eclipse.listener.subject.listeners";
		
	private static Subject instance = null;
	private Vector listeners = null;
	private int state = 0;
	
	public Subject(IPluginDescriptor descriptor) {
		super(descriptor);
		instance = this;
	}
	
	/**
	 * Update the state of the subject. Causes notifications
	 * to be broadcast to the subject's listeners.
	 */	
	public void update() {
		state++;
		System.out.println("subject updated to " + state);
		notifyListeners();
	}

	/**
	 * Notify each member of each extension of the <i>listeners</i>
	 * extension point of the update. For each member, an instance of
	 * the extender-supplied custom <i>listener</i> callback class is notified
	 * via its <i>IListener.listen()</i> method.
	 */
	private void notifyListeners() {
		Vector listeners = getListeners();
			
		for (Enumeration all = listeners.elements(); all.hasMoreElements(); ) {
			IListener listener = (IListener) all.nextElement();
			listener.listen();
		}
	}
	
	/**
	 * Get the list of the members of the <i>listeners</i> extension-point.
	 * The first time this method is called, extension processing takes
	 * place and instantiates listeners for each <i>listeners</i>
	 * member.
	 * 
	 * @return List of <i>listeners</i> callback objects.
	 */
	private final Vector getListeners() {
		
		if (listeners != null) {
			return listeners;
		}
		
		listeners = new Vector();		
		IPluginRegistry registry = Platform.getPluginRegistry();
		IExtensionPoint extensionPoint = registry.getExtensionPoint(LISTENERS_EXTENSION_POINT);
		IExtension[] extensions = extensionPoint.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			IExtension extension = extensions[i];
			IConfigurationElement[] elements = extension.getConfigurationElements();
			for (int j = 0;  j < elements.length; j++) {
				try {
					IConfigurationElement configurationElement = elements[j];

					Object listener = configurationElement.createExecutableExtension("class");
					if (listener instanceof IListener) {
						listeners.addElement(listener);
					}
				}
				catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}				
		return listeners;
	}
}
